/*


Skinned vertex shader

*/

bind modelViewProjectionMatrix to "mvp"
bind normalMatrix to "normalMatrix"

bind position to "position"
bind normal to "normal"
bind matrix_weights to "weights"
bind matrix_indices to "bone_indices"
bind matrix_palette to "matrix_palette"
bind texCoord to "texCoord"

#define NUMBER_OF_MATRICES	48

const int c_zero = 0;
const int c_one = 1;
const int c_two = 2;
const int c_three = 3;

attribute vec4 position;
attribute vec3 normal;
attribute vec4 matrix_weights;
attribute vec4 matrix_indices;
attribute vec2 texCoord;

varying vec2 v_texCoord;
varying vec4 colorVarying;

uniform mat4 modelViewProjectionMatrix;
uniform mat3 normalMatrix;

// Store NUMBER_OF_MATRICES 4x3 matrices as array of floats representing each matrix in row-major order ie/ 3 vec4s
	
uniform vec4 matrix_palette[NUMBER_OF_MATRICES * 3];	

void skin_position(in vec4 position, float m_wt, int m_index, inout vec4 skinned_position)
{
	vec4	temp;
	temp.x = dot(position, matrix_palette[m_index]);
	temp.y = dot(position, matrix_palette[m_index + c_one]);
	temp.z = dot(position, matrix_palette[m_index + c_two]);
	temp.w = position.w;
	skinned_position += m_wt * temp;
}

void skin_normal(in vec3 normal, float m_wt, int m_index, inout vec3 skinned_normal)
{
	vec3 temp;
	temp.x = dot(normal, matrix_palette[m_index].xyz);
	temp.y = dot(normal, matrix_palette[m_index + c_one].xyz);
	temp.z = dot(normal, matrix_palette[m_index + c_two].xyz);
	skinned_normal += m_wt * temp;
}

void do_skinning(in vec4 position, in vec3 normal, inout vec4 skinned_position, inout vec3 skinned_normal)
{
	float m_wt = matrix_weights[0];
	int m_index;

	if (m_wt > 0.0)
	{
		m_index = int(matrix_indices[0]) * c_three;
		skin_position(position, m_wt, m_index, skinned_position);
		skin_normal(normal, m_wt, m_index, skinned_normal);
	}
	
	m_wt = matrix_weights[1];
	if (m_wt > 0.0)
	{
		m_index = int(matrix_indices[1]) * c_three;
		skin_position(position, m_wt, m_index, skinned_position);
		skin_normal(normal, m_wt, m_index, skinned_normal);
	}

	m_wt = matrix_weights[2];
	if (m_wt > 0.0)
	{
		m_index = int(matrix_indices[2]) * c_three;
		skin_position(position, m_wt, m_index, skinned_position);
		skin_normal(normal, m_wt, m_index, skinned_normal);
	}

	m_wt = matrix_weights[3];
	if (m_wt > 0.0)
	{
		m_index = int(matrix_indices[3]) * c_three;
		skin_position(position, m_wt, m_index, skinned_position);
		skin_normal(normal, m_wt, m_index, skinned_normal);
	}
}

void main()
{
	vec4 skinned_position = vec4(float(c_zero));
	vec3 skinned_normal = vec3(float(c_zero));

	do_skinning(position, normal, skinned_position, skinned_normal);

	vec3 eyeNormal = normalize(normalMatrix * skinned_normal);
	vec3 lightPosition = vec3(0.0, 0.0, 1.0);
	vec4 diffuseColor = vec4(0.4, 0.4, 1.0, 1.0);

	float nDotVP = max(0.0, dot(eyeNormal, normalize(lightPosition)));
	colorVarying = diffuseColor * nDotVP;
	gl_Position = modelViewProjectionMatrix * skinned_position;
	v_texCoord = texCoord;
}